/**
 * \file output/proticdbml.cpp
 * \date 11/5/2017
 * \author Olivier Langella
 * \brief PROTICdbML writer
 */

/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "proticdbml.h"
#include <pappsomspp/pappsoexception.h>
#include <pappsomspp/utils.h>
#include <pappsomspp/grouping/grpprotein.h>
#include <pappsomspp/amino_acid/Aa.h>
#include "../core/msrun.h"

ProticdbMl::ProticdbMl(const QString & out_filename)
{
    //_p_digestion_pipeline = p_digestion_pipeline;

    //_mzidentml = "http://psidev.info/psi/pi/mzIdentML/1.1";
    QString complete_out_filename = out_filename;
    _output_file = new QFile(complete_out_filename);

    if (_output_file->open(QIODevice::WriteOnly))
    {
        _output_stream = new QXmlStreamWriter();
        _output_stream->setDevice(_output_file);
    } else
    {
        throw pappso::PappsoException(QObject::tr("error : cannot open the PROTICdbML output file : %1\n").arg(out_filename));
    }

    _output_stream->setAutoFormatting(true);
    _output_stream->writeStartDocument("1.0");


}

ProticdbMl::~ProticdbMl()
{
    delete _output_file;
    delete _output_stream;
}

void ProticdbMl::close() {
    _output_stream->writeEndDocument();
    _output_file->close();
}



void ProticdbMl::write(ProjectSp sp_project) {

    qDebug() << "ProticdbMl::write begin";
    _duracel.start();
    _sp_project = sp_project;
    if (_sp_project.get() == nullptr) {
        throw pappso::PappsoException(QObject::tr("Error writing PROTICdbML file :\n project is empty"));
    }


    _output_stream->writeNamespace("http://www.w3.org/2001/XMLSchema-instance","xsi");
    // writer.setDefaultNamespace(namespaceURI);
    _output_stream->writeStartElement("PROTICdb");
    //_output_stream->writeAttribute("xmlns","http://pappso.inra.fr/xsd/masschroqml/2.2");
    //_output_stream->writeAttribute("http://www.w3.org/2001/XMLSchema-instance","schemaLocation","http://pappso.inra.fr/xsd/masschroqml/2.2 http://pappso.inra.fr/xsd/masschroq-2.2.xsd");


    //writer.writeAttribute(xmlnsxsi, "noNamespaceSchemaLocation", xsischemaLocation);

    _output_stream->writeAttribute("version", "1.0");

    _output_stream->writeAttribute("type", "MSidentificationResults");

    writeIdentMethod();
    _output_stream->writeStartElement("sequences");
    for (IdentificationGroup * p_ident_group : sp_project.get()->getIdentificationGroupList()) {

        for (ProteinMatch * p_protein_match: p_ident_group->getProteinMatchList()) {
            writeSequence(p_protein_match);
        }
    }
    //</sequences>
    _output_stream->writeEndElement();

    writeProject();

}

void ProticdbMl::writeSequence(ProteinMatch * p_protein_match) {
    qDebug() << "ProticdbMl::writeSequence begin";
    if (p_protein_match->getValidationState() != ValidationState::grouped) return;
    pappso::GrpProtein * p_grp_protein = p_protein_match->getGrpProteinSp().get();

    if (_map_accession2xmlid.find(p_grp_protein->getAccession()) == _map_accession2xmlid.end()) {
        //not found

        QString id = QString("seq%1").arg(_map_accession2xmlid.size());
        _output_stream->writeStartElement("sequence");
        _output_stream->writeAttribute("id", id);
        QString display_id =  QString("%1 %2").arg(p_grp_protein->getAccession()).arg(p_protein_match->getProteinXtpSp().get()->getDescription());
        _output_stream->writeAttribute("display_id", display_id.left(60));
        // <dbxref key="AT5G16390"
        // dbname="AGI_LocusCode"></dbxref>
        //if (prot.get_dbxref_type().equals("no") == false) {
        for (const std::pair<ExternalDatabase, QString> & dbxref :p_protein_match->getProteinXtpSp().get()->getDbxrefList()) {
            //if (false) {
            _output_stream->writeStartElement("dbxref");
            _output_stream->writeAttribute("dbname",
                                           Utils::getDatabaseName (dbxref.first));
            _output_stream->writeAttribute("key",
                                           dbxref.second);
            _output_stream->writeEndElement();// dbxref
        }
        _output_stream->writeStartElement("description");
        _output_stream->writeCharacters(p_protein_match->getProteinXtpSp().get()->getDescription());
        _output_stream->writeEndElement();// description
        _output_stream->writeStartElement("seq");
        _output_stream->writeCharacters(p_protein_match->getProteinXtpSp().get()->getSequence());
        _output_stream->writeEndElement();// seq

        _output_stream->writeEndElement();// sequence

        _map_accession2xmlid.insert(std::pair<QString, QString>(p_grp_protein->getAccession(), id));

    }
    qDebug() << "ProticdbMl::writeSequence end";
}

void ProticdbMl::writeProject() {
qDebug() << "ProticdbMl::writeProject begin";
    _output_stream->writeStartElement("project");
    _output_stream->writeAttribute("name", "");
    _output_stream->writeAttribute("id", "p1");
    // ajout des echantillons_msrun
    writeSamples();
    writeMsRuns();

    _output_stream->writeStartElement("identificationRuns");

    for (IdentificationGroup * p_identification : _sp_project.get()->getIdentificationGroupList()) {
        _output_stream->writeStartElement("identificationRun");
        // ajout d'une mnip d'identification
        writeIdentificationRun(p_identification);

        // ajout des petides
        writepeptideHits(p_identification);

        // ajout des mathces
        writeMatchs(p_identification);

        _output_stream->writeEndElement();// identificationRun
    }
    _output_stream->writeEndElement();// identificationRuns
    _output_stream->writeEndElement();// project

    _output_stream->writeEndDocument();


qDebug() << "ProticdbMl::writeProject end duration = " << duracel << "ms";
}

void ProticdbMl::writeIdentMethod() {
        // Ajout des mÃ©thodes
        _output_stream->writeStartElement("identMeth");
        _output_stream->writeAttribute("name", "");
        _output_stream->writeAttribute("id", "m1");
        _output_stream->writeEndElement();

        // Ajout des base de donnÃ©es
        _output_stream->writeStartElement("customDb");
        _output_stream->writeAttribute("name", "");
        _output_stream->writeAttribute("id", "customdb0");
        _output_stream->writeEndElement();


}

void ProticdbMl::writeSamples() {
        _output_stream->writeStartElement("samples");
        for (std::vector<MsRunSp> msrun_sp : _sp_project.get()->getMsRunStore().getMsRunList()) {
                _output_stream->writeStartElement("sample");
                QString id_samp = "samp" + pappso::Utils::getLexicalOrderedString(sample_to_id.size());
                QString id_msrun = msrun_sp.get()->getXmlId();
                QString name = msrun_sp.get()->getSampleName();
                // balise sample
                _output_stream->writeAttribute("name", name);
                _output_stream->writeAttribute("id", id_samp);
                _output_stream->writeEmptyElement("description");
                _sample_to_id.insert(std::pair<QString, QString>( name, id_samp));

                // Element spectrumList = doc.createElement("spectrumList");
                // msRun.appendChild(spectrumList);

                //msrun_to_id.put(samp.getSampleName(), id_msrun);
                _output_stream->writeEndElement();// sample
            
        }
        _output_stream->writeEndElement();// samples
}

void ProticdbMl::writeMsRuns() {
    
        _output_stream->writeStartElement("msRuns");
        for (std::vector<MsRunSp> msrun_sp : _sp_project.get()->getMsRunStore().getMsRunList()) {

                QString name = msrun_sp.get()->getSampleName();

                QString id_samp = _sample_to_id.at(name);

                // Element spectrumList = doc.createElement("spectrumList");
                // msRun.appendChild(spectrumList);

                QString id_msrun = msrun_sp.get()->getXmlId();

                // balise MsRun
                _output_stream->writeStartElement("msRun");
                _output_stream->writeAttribute("sample_id", id_samp);
                _output_stream->writeAttribute("id", id_msrun);

                _output_stream->writeStartElement("description");
                _output_stream->writeStartElement("admin");
                // writer.writeStartElement("contact");
                // writer.writeStartElement("email");
                // writer.writeCharacters("valot@moulon.inra.fr");
                // writer.writeEndElement();// email

                // writer.writeStartElement("name");
                // writer.writeCharacters("Valot Benoit");
                // writer.writeEndElement();// name
                // writer.writeEndElement();// contact

                _output_stream->writeStartElement("sourceFile");
                _output_stream->writeStartElement("nameOfFile");
                _output_stream->writeCharacters(name + ".RAW");
                _output_stream->writeEndElement();// nameOfFile
                _output_stream->writeEndElement();// sourceFile

                _output_stream->writeStartElement("sampleName");
                _output_stream->writeCharacters(name);
                _output_stream->writeEndElement();// sampleName
                _output_stream->writeEndElement();// admin
                _output_stream->writeEndElement();// description

                _output_stream->writeEndElement();// msRun
            
        }
        _output_stream->writeEndElement();// msRuns
}


void ProticdbMl::writeIdentificationRun(IdentificationGroup * p_identification) {
        _output_stream->writeAttribute("ident_method_id", "m1");
        _output_stream->writeAttribute("customdb_id", "customdb0");

        _output_stream->writeStartElement("description");
        _output_stream->writeStartElement("admin");
        // writer.writeStartElement("contact");
        // writer.writeStartElement("email");
        // writer.writeCharacters("valot@moulon.inra.fr");
        // writer.writeEndElement();// email

        // writer.writeStartElement("name");
        // writer.writeCharacters("Valot Benoit");
        // writer.writeEndElement();// name
        // writer.writeEndElement();// contact
        /*
        this.xpipFile = XtandemPipelineSession.getInstance()
        .getCurrentXpipFile();

        _output_stream->writeStartElement("sourceFile");
        _output_stream->writeStartElement("nameOfFile");
        // TODO
        // get the loaded xpip file name
        _output_stream->writeCharacters(this.xpipFile.getName());
        _output_stream->writeEndElement();// nameOfFile
        _output_stream->writeStartElement("pathToFile");
        _output_stream->writeCharacters(this.xpipFile.getAbsolutePath());
        _output_stream->writeEndElement();// pathToFile
        _output_stream->writeStartElement("fileType");
        _output_stream->writeCharacters("XPIP file");
        _output_stream->writeEndElement();// fileType
        _output_stream->writeEndElement();// sourceFile
*/
        _output_stream->writeEndElement();// admin

        // if we can retrieve original informations in xml xtandem results
        HashSet<IdentificationXtandemFile> xtSourceFiles = Utils
        .getXtandemFileList(identification);
        if (xtSourceFiles != null) {
            //
            for (IdentificationXtandemFile xtFile : xtSourceFiles) {
                xtFile.writeProticDbMlDataProcessing(this.writer);
            }
        } else {
            writer.writeStartElement("dataProcessing");
            writer.writeStartElement("software");
            writer.writeStartElement("name");
            writer.writeAttribute("acc", "PROTICdbO:0000283");
            writer.writeCharacters("X!Tandem");
            writer.writeEndElement();// name
            writer.writeStartElement("version");
            writer.writeCharacters(XtandemPipelineSession.getInstance()
            .getConfig().getXtandemVersion());
            writer.writeEndElement();// version
            writer.writeEndElement();// software
            writer.writeStartElement("processingMethod");
            writer.writeEndElement();// processingMethod
            writer.writeEndElement();// dataProcessing
        }

        // id: PROTICdbO:0000316
        // name: X!TandemPipeline

        writer.writeStartElement("dataProcessing");
        writer.writeStartElement("software");
        writer.writeStartElement("name");
        writer.writeAttribute("acc", "PROTICdbO:0000316");
        writer.writeCharacters("X!TandemPipeline");
        writer.writeEndElement();// name
        writer.writeStartElement("version");
        writer.writeCharacters(XtandemPipelineMain.version);
        writer.writeEndElement();// version
        writer.writeEndElement();// software
        writer.writeStartElement("processingMethod");
        // cvParams

        writeCvParam("PROTICdbO:0000323", ""
        + XtandemPipelineSession.getInstance().getConfig()
        .get_protein_evalue(),
        "X!TandemPipeline filter on protein evalue (log)");

        // indi
        // combine
        // phospho
        xtpExperimentType = XtandemPipelineSession.getInstance()
        .getDataTypeBase().getType();
        if (xtpExperimentType.equals("indiv")) {
            this.writeCvParam("PROTICdbO:0000319", xtpExperimentType, "");
        }
        if (xtpExperimentType.equals("combi")) {
            this.writeCvParam("PROTICdbO:0000320", xtpExperimentType, "");
        }
        if (xtpExperimentType.equals("phospho")) {
            this.writeCvParam("PROTICdbO:0000321", xtpExperimentType, "");
        }

        this.writeCvParam("PROTICdbO:0000325", ""
        + XtandemPipelineSession.getInstance().getConfig()
        .get_peptide_number(),
        "X!TandemPipeline filter on minimal number of peptide per protein");

        this.writeCvParam("PROTICdbO:0000324", ""
        + XtandemPipelineSession.getInstance().getConfig()
        .get_peptide_evalue(),
        "X!TandemPipeline filter on peptide evalue");

        // TODO write database filter
        // this.writeCvParam("PROTICdbO:0000324", ""
        // +
        // XtandemPipelineSession.getInstance().getConfig().get_database_filter(),
        // "X!TandemPipeline filter on peptide evalue");

        writer.writeEndElement();// processingMethod

        writer.writeEndElement();// dataProcessing

        writer.writeEndElement();// description
}

private void writePeptideHit(Peptide pep, Group group,
                             PeptideMass peptideMass) throws XMLStreamException, MSMSException {

    if (peptideMass == null) {
        String message = "peptideMass == null";
        logger.error(message);
        throw new MSMSException(message);
    }

    String pep_hit_id = "pep" + pepSampleScanCle2xmlId.size();
    // param par default
    writer.writeStartElement("peptideHit");
    writer.writeAttribute("calc_mr", "" + (pep.get_mhplus_theo() - 1.00794));
    double exp_mz = (pep.get_mhplus_obser() + (1.00794 * (pep.getCharge() - 1)))
    / pep.getCharge();
    writer.writeAttribute("exp_mz", "" + exp_mz);
    writer.writeAttribute("delta", "" + pep.get_deltamass());
    writer.writeAttribute("exp_mr", "" + (pep.get_mhplus_obser() - 1.00794));
    writer.writeAttribute("acq_number", "" + pep.get_scan());
    writer.writeAttribute("ms_run_id",
    "" + msrun_to_id.get(pep.get_sample()));
    writer.writeAttribute("id", pep_hit_id);
    writer.writeAttribute("exp_z", "" + pep.getCharge());

    this.writeCvParam("PROTICdbO:0000339",
    Utils.getPappsoPeptideMassId(group, peptideMass),
    "peptide mass id");

    if (xtpExperimentType.equals("phospho")) {
        this.writeCvParam("PROTICdbO:0000349",
        Utils.getPappsoPhosphoPeptideMassId(group, peptideMass),
        "phosphopeptide mass id");
    }

    // pep.getMsRun().
    IdentificationDataSource idDataSource = pep
    .getIdentificationDataSource();
    if (idDataSource.getClass().equals(IdentificationXtandemFile.class)) {
        // cvparam specifique xtandem

        this.writeCvParam("PROTICdbO:0000287", "" + pep.get_evalue(),
        "xtandem peptide evalue");

        this.writeCvParam("PROTICdbO:0000288", "" + pep.get_hypercorr(),
        "xtandem peptide hyperscore");
    }

    this.writeCvParam("PROTICdbO:0000289",
    "" + pep.getRetentionTimeInMinutes(),
    "xtandem peptide retention time");
    /*
     * writer.writeStartElement("cvParam"); writer.writeAttribute("value",
     * "" + pep.get_RT()); writer.writeAttribute("name",
     * "xtandem peptide retention time"); writer.writeAttribute("cvLabel",
     * "PROTICdbO"); writer.writeAttribute("accession",
     * "PROTICdbO:0000289"); writer.writeEndElement();// cvParam
     */

    // Element cvParam4 = doc.createElement("cvParam");
    // cvParam4.setAttribute("value", "" + pep.get_post());
    // cvParam4.setAttribute("name", "xtandem peptide post");
    // cvParam4.setAttribute("cvLabel", "PROTICdbO");
    // cvParam4.setAttribute("accession", "PROTICdbO:00000XX");
    // peptideHit.appendChild(cvParam4);
    //
    // Element cvParam5 = doc.createElement("cvParam");
    // cvParam5.setAttribute("value", "" + pep.get_pre());
    // cvParam5.setAttribute("name", "xtandem peptide pre");
    // cvParam5.setAttribute("cvLabel", "PROTICdbO");
    // cvParam5.setAttribute("accession", "PROTICdbO:00000XX");
    // peptideHit.appendChild(cvParam5);

    // sequences avec les modifs
    writer.writeStartElement("pepSeq");
    writer.writeStartElement("peptide");
    writer.writeCharacters("" + pep.getSequence());
    writer.writeEndElement();// peptide
    if (pep.get_Modifs().size() != 0) {
        this.writePtms(pep.get_Modifs());
    }
    writer.writeEndElement();// pepSeq

    pepSampleScanCle2xmlId.put(pep.get_sample_scan_sequencepeptide_cle(),
    pep_hit_id);
    writer.writeEndElement();// peptideHit
}

private void writePtms(ArrayList<Modifs> mods) throws XMLStreamException {
    writer.writeStartElement("ptms");
    for (Modifs mod : mods) {
        writer.writeStartElement("ptm");
        writer.writeAttribute("diff_mono", "" + mod.get_modvalue());
        writer.writeAttribute("position", "" + mod.get_posi());
        writer.writeAttribute("aa", "" + mod.get_AA());
        try {
            this.writeOboModif(mod.getPsiMod());
        } catch (MSMSException e) {
            String message = "error writing OBO modif : " + e.getMessage();
            logger.warn(message);
        }
        writer.writeEndElement();// ptm

    }
    writer.writeEndElement();// ptms
}

void ProticdbMl::writeOboModif(pappso::AaModificationP mod) {
    _output_stream->writeStartElement("cvParam");
    _output_stream->writeAttribute("name", mod->getName());
    _output_stream->writeAttribute("cvLabel", "MOD");
    _output_stream->writeAttribute("accession", mod->getAccession());
    _output_stream->writeEndElement();// cvParam
}

private void writepeptideHits(Identification identification)
throws MSMSException {
    try {
        writer.writeStartElement("peptideHits");
        for (Group group : identification.getGrouping().getGroupList()) {
            // generating peptide numbers
            PeptideMassSet pepMassSet = new PeptideMassSet();

            Hashtable<String, PeptideMass> pepSampleScanCle2peptideMass;
            if (this.xtpExperimentType.equals("phospho")) {
                pepSampleScanCle2peptideMass = pepMassSet
                .numberingPhosphoGetSampleScanCle2peptideMass(group);

            } else {
                pepSampleScanCle2peptideMass = pepMassSet
                                               .numberingGetSampleScanCle2peptideMass(group);
            }

            for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
                for (HashSampleScanSetProt hashProt : sg
                        .getHashSampleScanSetProtList()) {

                    ArrayList<Peptide> peptideList = null;
                    if (this.xtpExperimentType.equals("phospho")) {
                        peptideList = ((Match) hashProt.getGrpProtMatch())
                                      .getValidPeptideInPhosphoIsland((HashSampleScanSetPhospho) hashProt);
                    } else {
                        peptideList = ((Match) hashProt.getGrpProtMatch())
                                      .get_valid_peptides();
                    }

                    for (Peptide pep : peptideList) {

                        if (pepSampleScanCle2xmlId.containsKey(pep
                                                               .get_sample_scan_sequencepeptide_cle()) == false) {

                            writePeptideHit(
                                pep,
                                group,
                                pepSampleScanCle2peptideMass.get(pep
                                                                 .get_sample_scan_sequencepeptide_cle()));

                        }
                    }
                }
            }
        }
        writer.writeEndElement();// "peptideHits");
    } catch (Exception e) {
        String message = "error writing peptideHits in ProticDbMl \n"
                         + e.getMessage();
        logger.error(message);
        throw new MSMSException(message);
    }
}

private void writeMatchs(Identification identification)
throws MSMSException {
    try {
        writer.writeStartElement("matchs");
        for (Group group : identification.getGrouping().getGroupList()) {
            for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
                writer.writeStartElement("match");

                // echantillons
                Hashtable<String, String> view = new Hashtable<String, String>();
                for (HashSampleScanSetProt hashProt : sg
                .getHashSampleScanSetProtList()) {
                    Match match = (Match) hashProt.getGrpProtMatch();
                    for (MsRun sample : match.get_sample_matchs()) {
                        if (!view.containsKey(sample.getSampleName())) {
                            writer.writeStartElement("matchSample");
                            writer.writeAttribute(
                                "sample_id",
                                ""
                                + sample_to_id.get(sample
                            .getSampleName()));
                            String name = sample.getSampleName();
                            name = name.replaceAll(".xml", "");
                            writer.writeAttribute("name", "" + name);
                            view.put(sample.getSampleName(), "ok");
                            writer.writeEndElement();// "matchSample");
                        }
                    }
                }

                // proteins
                int k = 1;
                for (HashSampleScanSetProt hashProt : sg
                        .getHashSampleScanSetProtList()) {
                    writer.writeStartElement("proteinHit");
                    Protein prot = (Protein) hashProt.getGrpProtMatch()
                                   .getGrpProtein();
                    writer.writeAttribute("sequence_id",
                                          "" + prot_to_id.get(prot.get_description()));
                    // proteinHit.setAttribute("score", "");
                    writer.writeAttribute("rank", "" + k);

                    // //cvparam
                    this.writeCvParam("PROTICdbO:0000284",
                                      "" + prot.get_mw(),
                                      "MW determination by Benuch");
                    /*
                     * writer.writeStartElement("cvParam");
                     * writer.writeAttribute("value", "" + prot.get_mw());
                     * writer.writeAttribute("name",
                     * "MW determination by Benuch");
                     * writer.writeAttribute("cvLabel", "PROTICdbO");
                     * writer.writeAttribute("accession",
                     * "PROTICdbO:0000284"); writer.writeEndElement();//
                     * cvParam
                     */
                    // evalue
                    MsRunSet samples = ((Match) hashProt.getGrpProtMatch())
                                       .get_sample_matchs();
                    this.writeCvParam("PROTICdbO:0000291",
                                      "" + prot.get_evalue(), "Xtandem log evalue");
                    /*
                     * writer.writeStartElement("cvParam");
                     * writer.writeAttribute("value", "" +
                     * prot.get_evalue());
                     *
                     * writer.writeAttribute("name", "Xtandem log evalue");
                     * writer.writeAttribute("cvLabel", "PROTICdbO");
                     * writer.writeAttribute("accession",
                     * "PROTICdbO:0000291"); writer.writeEndElement();//
                     * cvParam
                     */

                    // coverage
                    if (samples.size() == 1) {
                        this.writeCvParam(
                            "PROTICdbO:0000285",
                            ""
                            + ((Match) hashProt
                               .getGrpProtMatch())
                            .get_coverage(),
                            "protein coverage");
                    }

                    // [Term]
                    // id: PROTICdbO:0000335
                    // name: X!TandemPipeline PAI
                    this.writeCvParam("PROTICdbO:0000335",
                                      "" + hashProt.getPAI(), "PAI");

                    // [Term]
                    // id: PROTICdbO:0000337
                    // name: protein group number
                    String pappsoGroupingNumber = Utils
                                                  .getPappsoGroupingNumber(group, sg, k);
                    if (pappsoGroupingNumber != null) {
                        this.writeCvParam("PROTICdbO:0000337",
                                          pappsoGroupingNumber, "grouping number");
                    }

                    writer.writeEndElement();// "proteinHit");
                    k++;
                }
                for (HashSampleScanSetProt hashProt : sg
                        .getHashSampleScanSetProtList()) {
                    Protein prot2 = (Protein) hashProt.getGrpProtMatch()
                                    .getGrpProtein();
                    // peptidesHitRef

                    ArrayList<Peptide> peptideList = null;
                    if (this.xtpExperimentType.equals("phospho")) {
                        peptideList = ((Match) hashProt.getGrpProtMatch())
                                      .getValidPeptideInPhosphoIsland((HashSampleScanSetPhospho) hashProt);
                    } else {
                        peptideList = ((Match) hashProt.getGrpProtMatch())
                                      .get_valid_peptides();
                    }

                    for (Peptide pep : peptideList) {
                        // for (Peptide pep : ((Match)
                        // hashProt.getGrpProtMatch())
                        // .get_peptide_order_inclus_redon()) {
                        String xmlId = pepSampleScanCle2xmlId.get(pep
                                       .get_sample_scan_sequencepeptide_cle());
                        if (xmlId == null) {
                            String message = "the xml of peptide "
                                             + pep.get_sample_scan_sequencepeptide_cle()
                                             + " is not known in pepSampleScanCle2xmlId";
                            logger.debug(message);
                            throw new Exception(message);
                        }
                        writer.writeStartElement("peptideHitRef");
                        writer.writeAttribute("peptide_hit_id", "" + xmlId);

                        writer.writeStartElement("fromSeq");
                        writer.writeAttribute(
                            "seq_id",
                            ""
                            + prot_to_id.get(prot2
                                             .get_description()));
                        writer.writeAttribute("start", "" + pep.get_start());
                        writer.writeAttribute("stop", "" + pep.get_stop());
                        String res1 = pep.get_pre();
                        res1 = res1.replaceAll("]", "-");
                        writer.writeAttribute("residue_before_nter", ""
                                              + res1.charAt((res1.length() - 1)));
                        String res2 = pep.get_post();
                        res2 = res2.replaceAll("]", "-");
                        writer.writeAttribute("residue_after_cter", ""
                                              + res2.charAt(0));

                        writer.writeEndElement();// fromSeq
                        writer.writeEndElement();// peptideHitRef
                    }

                }
                writer.writeEndElement();// "match");
            }
        }
        writer.writeEndElement();// "matchs");
    } catch (Exception e) {
        String message = "error writing Matchs in ProticDbMl \n"
                         + e.getMessage();
        logger.error(message);
        throw new MSMSException(message);
    }
}

void ProticdbMl::writeCvParam(QString acc, QString value, QString description) {
    _output_stream->writeStartElement("cvParam");

    _output_stream->writeAttribute("value",value);
    if (description.isEmpty()) {
        _output_stream->writeAttribute("name", "N.A.");
    } else {
        _output_stream->writeAttribute("name", description);
    }
    _output_stream->writeAttribute("cvLabel", "PROTICdbO");
    _output_stream->writeAttribute("accession", acc);
    _output_stream->writeEndElement();// cvParam

}
